# Generated by Django 3.2.15 on 2023-12-16 20:16

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Indexes',
            fields=[
                ('guid', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('document', models.CharField(db_index=True, editable=False, max_length=255, verbose_name='Document name')),
                ('model', models.CharField(db_index=True, editable=False, max_length=255, verbose_name='Model name')),
                ('updated_at', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Date and time of last index')),
                ('hints', models.PositiveIntegerField(default=0, editable=False, verbose_name='Count of hints in index')),
            ],
            options={
                'verbose_name': 'Index',
                'verbose_name_plural': 'Indexes',
            },
        ),
    ]



from datetime import datetime

from dateutil.tz import tz
from django.apps import apps
from django.conf import settings
from django.contrib import admin, messages
from django.utils.translation import gettext as _
from django_solr.models import Indexes
from haystack import connections as haystack_connections
from haystack.management.commands.update_index import do_update

from django_solr import models


class IndexesAdmin(admin.ModelAdmin):
    list_display = ("document", "model", "hints", "updated_at")
    readonly_fields = ("document", "model", "hints", "updated_at")
    actions = ("clear_indexes", "rebuild_indexes", "update_indexes")

    def get_index(self, backend_name: str, model):
        unified_index = haystack_connections[backend_name].get_unified_index()
        return unified_index.get_index(model)

    def backends(self) -> str:
        for backend_name in haystack_connections.connections_info.keys():
            yield backend_name

    def has_add_permission(self, request) -> bool:
        return False

    def has_change_permission(self, request, obj=...) -> bool:
        return False

    def has_delete_permission(self, request, obj=...) -> bool:
        return False

    def _clear_index(self, queryset):
        models_to_delete = []
        record: Indexes
        for record in queryset:
            app_label, model_name = record.model.split(".")
            models_to_delete.append(apps.get_model(app_label, model_name))
            record.hints = 0
            record.updated_at = None
        for backend_name in self.backends():
            backend = haystack_connections[backend_name].get_backend()
            backend.clear(models=models_to_delete, commit=True)
        Indexes.objects.bulk_update(queryset, ["hints", "updated_at"])

    @admin.action(description=_("Clear selected indexes"))
    def clear_indexes(self, request, queryset):
        try:
            self._clear_index(queryset)
            messages.add_message(request=request, level=messages.INFO, message=_("Successfully cleared"))
        except Exception as error:
            error_msg = _(f"Clearing error occured")
            messages.add_message(request=request, level=messages.ERROR, message=f"{error_msg}: {error}")

    @admin.action(description=_("Rebuild selected indexes"))
    def rebuild_indexes(self, request, queryset):
        try:
            self._clear_index(queryset)
            self._update_index(queryset)
            messages.add_message(request=request, level=messages.INFO, message=_("Successfully rebuilt"))
        except Exception as error:
            error_msg = _(f"Rebuilding error occured")
            messages.add_message(request=request, level=messages.ERROR, message=f"{error_msg}: {error}")

    def do_update(self, backend, queryset):
        record: Indexes
        if settings.USE_TZ:
            tz_info = tz.gettz(settings.TIME_ZONE)
        else:
            tz_info = None
        for record in queryset:
            app_label, model_name = record.model.split(".")
            model = apps.get_model(app_label, model_name)
            index = self.get_index(backend.connection_alias, model)
            qs = index.build_queryset(using=backend.connection_alias)
            total = qs.count()
            batch_size = backend.batch_size
            max_pk = None
            for start in range(0, total, batch_size):
                end = min(start + batch_size, total)
                max_pk = do_update(backend, index, qs, start, end, total, last_max_pk=max_pk)
            record.hints = index.search().count()
            record.updated_at = datetime.now(tz=tz_info)

    def _update_index(self, queryset):
        for backend_name in self.backends():
            backend = haystack_connections[backend_name].get_backend()
            self.do_update(backend, queryset)
            Indexes.objects.bulk_update(queryset, ["hints", "updated_at"])

    @admin.action(description=_("Update selected indexes"))
    def update_indexes(self, request, queryset):
        try:
            self._update_index(queryset)
            messages.add_message(request=request, level=messages.INFO, message=_("Successfully updated"))
        except Exception as error:
            error_msg = _(f"Update error occured")
            messages.add_message(request=request, level=messages.ERROR, message=f"{error_msg}: {error}")


if admin.site.is_registered(models.Indexes):
    admin.site.unregister(models.Indexes)
admin.site.register(models.Indexes, IndexesAdmin)



from django.db import models as django_models
from django.db.models.fields import NOT_PROVIDED as DJANGO_NOT_PROVIDED
from haystack.constants import DJANGO_ID, DOCUMENT_FIELD, ID
from haystack.indexes import *

from .backend import SolrSearchQuerySet
from .exception import MultipleResultError, NotFoundError
from .fields import NestedField, TextGeneralField


class SolrDocument(SearchIndex):
    """
    Основной класс объявления SOLR-индексов (документов).
    Т.к. при традиционном подходе к поиску основным поисковым полем является
    единственное - text, оно объявляется по умолчанию для всех документов.
    При этом, оно не является обязательным, т.к. исключена проверка наличия шаблона.

    Для связи с моделью Django необходимо объявить вложенный класс `Django` с атрибутом `model`,
    в значении которого указать конкретную модель из включенного в проект приложения.

    Также добавлен новый тип поля - NestedField, представляющее собой вложенную иерархию.
    Обязательность полей вложенных объектов определяется одноименными полями модели Django.

    Для объявления новых документов нужен атрибут `haystack_use_for_indexing` в значении
    `True` (указать принудительно или использовать при наследовании класс
    haystack.indexes.Indexable).

        class MyDocument(SolrDocument, indexes.Indexable):
            class Django:
                model = models.MyModel


    ### ПРИМЕЧАНИЕ:

    Весь функционал и именование максимально приведены к идентичности с DSL OpenSearch, однако,
    без особенностей не обошлось:

    * Для полей первого уровня (объявленных непосредственно в классе документа) недопустимо
      использование имени поля `pk`, т.к. оно используется для сборки экземпляров
      `SolrSearchResult`. При этом, при индексации ошибка выброшена не будет, но возникнут
      проблемы при получении и обработке результатов запроса.
    * Для полей всех уровней не рекомендуется использовать имя атрибута `id`, т.к. значение
      идентификатора для каждого объекта индекса должно быть уникально в рамках ядра. Поэтому
      данное поле является составным и автогенерируемым. В его значение включется название
      приложения, модели и значение первичного ключа, разделенные символом точки. Натуральное
      значение первичного ключа модели хранится в служебном атрибуте django_id.

    """

    text = TextGeneralField(document=True, use_template=True)

    def __init__(self):
        super().__init__()
        self.model = None
        self._django = getattr(self, "Django", None)

        if self._django:
            self.model = getattr(self._django, "model", None)
            self.set_fields_attributes(self.model, self.fields)

    def get_model_fields_map(self, model) -> dict:
        _map = {}
        for field in model._meta.fields:
            _map[field.name] = field
            if field.primary_key:
                _map["pk"] = field
        return _map

    def set_fields_attributes(self, model, fields):
        """
        Автоматическая установка атрибутов объявленных полей документа.
        Применяются следующие правила:

        - `model_attr` (соответствие атрибуту индексируемой модели): по умолчанию
          используется имя атрибута, которому присваивается объявляемое поле.
          Возможно указание произвольного имени, но при этом в одноименном параметре
          ожидается имя поля, с которым данный атрибут связывается.

          * ```code = fields.TextField()```
            Объявленное текстовое поле `code` будет связано с одноименным полем
            индексируемой модели.

          * ```my_custom_code_field = fields.TextField(model_attr="code")```
            Объявленное текстовое поле `my_custom_code_field` будет связано
            с полем `code` индексируемой модели, при этом в индексе ключ будет
            именоваться как `my_custom_code_field`.

        - `null` (обязательность наличия значения): соответствует атрибуту `null`
          связанного поля модели.

        - В атрибут `model_field` указывается ссылка на соответствущее атрибуту
          документа поле индексируемой модели.

        - Если не указано значение по умолчанию, то используется значение
          по умолчанию для связанного поля модели, если оно также задано.

        """
        model_fields_map = self.get_model_fields_map(model)
        for field_name, field in fields.items():
            if field_name == DOCUMENT_FIELD:
                continue
            if isinstance(field, NestedField):
                relation_field = getattr(model, field_name, None)
                if not relation_field and field.model_attr:
                    relation_field = getattr(model, field.model_attr, None)
                if not relation_field:
                    continue
                related_model = relation_field.field.related_model
                if related_model is model and hasattr(relation_field, "rel"):
                    related_model = relation_field.rel.related_model
                self.set_fields_attributes(
                    model=related_model,
                    fields=field.properties,
                )
                setattr(field, "model_field", relation_field)
                if not field.has_default():
                    if isinstance(relation_field.field, django_models.ForeignKey):
                        field._default = dict()
                    elif isinstance(relation_field.field, django_models.ManyToManyField):
                        field.is_multivalued = True
                        field._default = list()
                    else:
                        # todo: Возможные варианты других типов полей
                        pass
                    continue

            if not field.model_attr:
                field.model_attr = field_name
            if field.model_attr in model_fields_map:
                field.null = model_fields_map[field.model_attr].null
            if model_field := model_fields_map.get(field.model_attr):
                setattr(field, "model_field", model_field)
            if not field.has_default():
                model_field_default = field.model_field.default
                if model_field_default is not DJANGO_NOT_PROVIDED and not callable(model_field_default):
                    field._default = model_field_default

    def get_model(self):
        return self.model

    @classmethod
    def search(cls) -> SolrSearchQuerySet:
        """
        Инициализировать запрос от текущей БД.
        """
        return SolrSearchQuerySet().models(cls.Django.model)

    def get_field_weights(self):
        return

    @classmethod
    def get(cls, *args, **kwargs):
        """
        Получить запись из индекса текущей модели по идентификатору.
        Допустимы имена первичных ключей: id, django_id, guid и uuid. Т.к. django_id -
        это эквивалент первичного ключа Django, если передан неименованный параметр,
        в качестве поискового ключа используется он.

        Ошибки:
            * `NotFoundError`: Запись не найдена.
            * `MultipleResultError`: Найдено больше, чем одна запись.

        """
        pk_fields = (ID, DJANGO_ID, "guid", "uuid")
        if args:
            pk_name = DJANGO_ID
            pk = args[0]
        else:
            for pk_name in pk_fields:
                if kwargs.get(pk_name):
                    pk = kwargs.get(pk_name)
                    break
            else:
                raise AttributeError(f"One of available pk names must be implemented: {', '.join(pk_fields)}")
        search = cls.search().filter(**{pk_name: pk})

        fields = kwargs.get("fields")
        if fields:
            search.fields(*fields)
        count = search.count()
        if not count:
            raise NotFoundError("No items found")
        if count > 1:
            raise MultipleResultError(f"Multiple records found ({count}). Expected only one.")
        return search.first()



from typing import List, Tuple, Union

from django.utils.encoding import force_str
from haystack.backends import SQ
from haystack.backends.solr_backend import (
    SolrEngine,
    SolrSearchBackend,
    SolrSearchQuery,
)
from haystack.constants import DEFAULT_ALIAS, DOCUMENT_FIELD
from haystack.exceptions import NotHandled
from haystack.models import SearchResult
from haystack.query import SearchQuerySet

from .fields import NestedField


class ProjectSolrSearchBackend(SolrSearchBackend):
    TYPE_MAP = {
        "date": "pdate",
        "datetime": "pdate",
        "integer": "plong",
    }

    def build_schema(self, fields) -> Tuple[str, List[dict]]:
        """Метод используется для составления схемы данных.
        Перегружен только в части маппинга типов полей."""
        content_field_name = ""
        schema_fields = []

        for _, field_class in fields.items():
            field_data = {
                "field_name": field_class.index_fieldname,
                "type": "text",
                "indexed": "true",
                "stored": "true",
                "multi_valued": "false",
            }

            if field_class.document is True:
                content_field_name = field_class.index_fieldname

            if field_class.field_type in self.TYPE_MAP:
                field_data["type"] = self.TYPE_MAP[field_class.field_type]
            else:
                field_data["type"] = field_class.field_type

            if field_class.is_multivalued:
                field_data["multi_valued"] = "true"

            if field_class.stored is False:
                field_data["stored"] = "false"

            # Do this last to override `text` fields.
            if field_class.indexed is False:
                field_data["indexed"] = "false"

                # If it's text and not being indexed, we probably don't want
                # to do the normal lowercase/tokenize/stemming/etc. dance.
                if field_data["type"] == "text_en":
                    field_data["type"] = "string"

            # If it's a ``FacetField``, make sure we don't postprocess it.
            if hasattr(field_class, "facet_for"):
                # If it's text, it ought to be a string.
                if field_data["type"] == "text_en":
                    field_data["type"] = "string"

            schema_fields.append(field_data)

        return content_field_name, schema_fields


class SolrSearchResult(SearchResult):
    """
    Класс представления одной записи результата запроса.
    """

    def __getattr__(self, attr):
        return self.__dict__.get(attr, None)

    def __getitem__(self, item):
        return getattr(self, item)

    @property
    def django_id(self):
        """
        Возвращает значение первичного ключа результирующей модели,
        приведенного к типу данных в соответствии с полем модели.
        """
        *_, django_id = self.id.split(".")
        return self.model._meta.pk.to_python(django_id)

    def _nested_to_dict(
        self, field, nested: Union[dict, List[dict], None], fields_only: bool
    ) -> Union[dict, List[dict], None]:
        if nested is None:
            if field.has_default():
                return field.default
            return None
        if isinstance(nested, list):
            return self._multiple_nested_to_dict(field, nested=nested, fields_only=fields_only)
        if isinstance(nested, dict):
            value = self._single_nested_to_dict(field, nested=nested, fields_only=fields_only)
            if field.is_multivalued:
                return [value]
            return value
        raise ValueError("Nested field value must be dict or list of dict type")

    def _single_nested_to_dict(self, field, nested: dict, fields_only: bool) -> dict:
        _dict = {}
        if not nested:
            return _dict
        for field_name, field_instance in field.properties.items():
            if fields_only and not self.is_need_dump_field(field_name):
                continue
            value = nested.get(field_name)
            if isinstance(field_instance, NestedField):
                _dict[field_name] = self._nested_to_dict(field_instance, nested=value, fields_only=fields_only)
            else:
                if value is None and field_instance.has_default():
                    value = field_instance.default
                _dict[field_name] = field_instance.convert(value)
                if hasattr(field_instance, "model_field") and field_instance.model_field.primary_key:
                    _dict[field_instance.model_field.name] = _dict[field_name]

        return _dict

    def _multiple_nested_to_dict(self, field, nested: List[dict], fields_only: bool) -> List[dict]:
        _list = list()
        if not nested:
            return _list
        for single_nested in nested:
            _list.append(self._single_nested_to_dict(field, single_nested, fields_only))
        return _list

    def is_need_dump_field(self, field_name: str) -> bool:
        if self._fields:
            return field_name in self._fields
        return True

    def to_dict(self, fields_inheritance: bool = False) -> dict:
        """
        Метод приведения записи к словарю.
        По умолчанию в результирующий словарь добавляется поле `pk`
        со значением первичного ключа модели и атрибут, соответствующий имени
        первичного ключа, например:

            ---------------------------------------------------------------------
            class ModelWithGuidPk(models.Model):
                guid = models.CharField(default=uuid.uuid4, primary_key=True)
                ...

            instance = ModelWithGuidPk(guid="dc5017dd-6299-4957-8d07-b9daa5947956")
            ...

            Экземпляр строки для строки для индексированного экземпляра `instance`
            будет преобразован в словарь:
            {
                "pk": "dc5017dd-6299-4957-8d07-b9daa5947956",
                "guid": "dc5017dd-6299-4957-8d07-b9daa5947956",
                ...
            }
            ---------------------------------------------------------------------

            class ModelWithIdPk(models.Model):
                id = models.BigAutoField(primary_key=True)

            instance = ModelWithIdPk(id=123)
            ...

            Экземпляр строки для строки для индексированного экземпляра `instance`
            будет преобразован в словарь:
            {
                "pk": "123",
                "guid": 123,
                ...
            }
            ---------------------------------------------------------------------

        :param fields_inheritance: Учитывать перечень выводимых полей (`fields`) для вложенных объектов.
        """
        if self._dict is None:
            from haystack import connections

            try:
                index = connections[DEFAULT_ALIAS].get_unified_index().get_index(self.model)
            except NotHandled:
                return {}

            model_pk_field = self.model._meta.pk

            self._dict = {
                "pk": self.pk,
                model_pk_field.attname: self.django_id,
            }
            for field_name, field in index.fields.items():
                if not self.is_need_dump_field(field_name):
                    continue
                value = getattr(self, field_name, None)
                if field_name == DOCUMENT_FIELD and not value:
                    continue
                if isinstance(field, NestedField):
                    self._dict[field_name] = self._nested_to_dict(field, nested=value, fields_only=fields_inheritance)
                else:
                    if value is None and field.has_default():
                        value = field.default
                    self._dict[field_name] = field.convert(value)
        return self._dict


class SolrSearchQuerySet(SearchQuerySet):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._internal_fields = ["id", "django_ct", "django_id", "score"]
        self._fields = []

    def fields(self, *fields):
        """
        Метод ограничения полей для вывода результатов.

        :param fields: Перечень имен полей для включения в ответ.
        """
        self._fields = list(fields)
        return self

    def _clone(self, klass=None):
        """
        Внутренний метод клонирования себя с сохранением полей для вывода.
        """
        clone = super()._clone(klass)
        clone._fields = self._fields
        return clone

    def first(self) -> SolrSearchResult:
        """Получить первый элемент выборки."""
        return self[0]

    def last(self) -> SolrSearchResult:
        """Получить последний элемент выборки."""
        return self[-1]

    def _fill_cache(self, start, end, **kwargs):
        if self._fields:
            query_fields = set(self._internal_fields)
            query_fields.update(self._fields)
            self.query.fields = list(query_fields)
        return super()._fill_cache(start, end, **kwargs)

    def nested(self, **kwargs):
        return self.narrow(ParentSQ(**kwargs))

    def nested_exists(self, nested: str):
        return self.narrow(ParentSQ(**{nested: None}))

    def post_process_results(self, results):
        """
        Метод пост-обработки результатов запроса.
        Метод перегружен присвоением атрибута `_fields` каждому результату для передачи полей,
        подлежащих включению в ответ.
        """
        results = super().post_process_results(results)
        for result in results:
            setattr(result, "_fields", self._fields)
        return results


class ProjectSolrSearchQuery(SolrSearchQuery):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.result_class = SolrSearchResult

    def build_params(self, spelling_query=None, **kwargs):
        kwargs = super().build_params(spelling_query, **kwargs)
        if "fields" not in kwargs:
            kwargs["fields"] = []
        kwargs["fields"].extend(["*", "score", "[child]"])
        return kwargs


class ProjectSolrEngine(SolrEngine):
    backend = ProjectSolrSearchBackend
    query = ProjectSolrSearchQuery


class EMPTY:
    pass


class ParentSQ(SQ):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._full_path = EMPTY
        self._path = EMPTY
        self._value = EMPTY
        self._key = EMPTY
        if self.children:
            self._parse_args()

    def __repr__(self):
        return "<%s: %s %s>" % (
            self.__class__.__name__,
            self.connector,
            self.as_query_string(self._repr_query_fragment_callback),
        )

    def _parse_args(self):
        self._full_path, self._value = self.children[0]
        if isinstance(self.value, list):
            self._value = " ".join(self._value)
        *path_items, self._key = self._full_path.split(".")
        self._path = "/" + "/".join(path_items)

    @property
    def value(self):
        if self.children and self._value is EMPTY:
            self._parse_args()
        return self._value

    @property
    def path(self):
        if self.children and self._path is EMPTY:
            self._parse_args()
        return self._path

    @property
    def key(self):
        if self.children and self._key is EMPTY:
            self._parse_args()
        return self._key

    def _repr_query_fragment_callback(self, field, filter_type, value):
        parent = '{!parent which="*:* -_nest_path_:*"}'
        nest_path = f"/{self.key}" if self.path == "/" else self.path
        queries = [f'+_nest_path_:"{nest_path}"']
        if value:
            queries.append(f"+{field}:({force_str(value)})")
        query = " ".join(queries)
        return f"{parent} ({query})"

    def as_query_string(self, *args, **kwargs):
        return self._repr_query_fragment_callback(self.key, self.connector, self.value)



from datetime import date, datetime
from typing import Union

from django.db import models
from django.template import TemplateDoesNotExist
from haystack.fields import DateField as _HaystackDateField
from haystack.fields import *  # noqa F403


class NestedField(SearchField):  # noqa F405
    field_type = "_nest_path_"

    def __init__(self, *args, **kwargs):
        self.properties = kwargs.pop("properties", {})
        super().__init__(*args, indexed=True, stored=True, null=True, **kwargs)

    def get_instances(self, obj, field_name: str) -> Union[models.QuerySet, models.Model, None]:
        """
        Получить связанные по полю объекты в зависимости от типа поля модели.
        Если связанный объект - объект модели (простая связь один-к-одному),
        то возвращается этот объект, если это менеджер (связь один-ко-многим,
        многие-ко-многим) то возвращается QuerySet по связанной модели.

        :param obj: Текущий индексируемый объект.
        :param field_name: Имя текущего поля связи.
        """
        set_field_name = f"{field_name}_set"
        if hasattr(obj, set_field_name):
            related = getattr(obj, set_field_name)
        elif not hasattr(obj, field_name):
            return None
        else:
            related = getattr(obj, field_name)
        if isinstance(related, models.manager.Manager):
            return related.all()
        if isinstance(related, models.Model):
            return related

    def _instance_to_dict(self, instance: models.Model, properties: dict) -> dict:
        doc = {}
        for property_name, field_class in properties.items():
            if not hasattr(instance, property_name):
                continue
            if not field_class.model_attr:
                field_class.model_attr = property_name
            doc[property_name] = field_class.prepare(instance)
        return doc

    def prepare(self, obj):
        """
        Представить объект в структуре данных для помещения в индекс.
        В качестве имени используется атрибут `model_attr`, указанный
        при инициализации поля, либо имя атрибута основной модели, если
        это вложенность первого уровня, или имя атрибута, если это вложенность
        более глубокого уровня.

        :param obj: Текущий индексируемый объект.
        """
        field_name = self.model_attr or self.instance_name
        related = self.get_instances(obj, field_name)
        if related:
            if isinstance(related, models.QuerySet):
                return [self._instance_to_dict(instance, self.properties) for instance in related]
            else:
                return self._instance_to_dict(related, self.properties)
        return None


class TextGeneralField(CharField):  # noqa F405
    field_type = "text_general"

    def prepare_template(self, obj):
        """
        Т.к. SOLR по умолчанию использует шаблоны для генерации текстового поля, основного
        для поиска при традиционном его использовании, требуется его указание в формате
        шаблонов Django. В данной реализации это требование исключено и в случае, когда шаблон
        отсутствует, поле text принимает пустое значение, а ошибка отсутствующего
        шаблона игнорируется.

        :param obj: Текущий индексируемый объект.
        """
        try:
            return super().prepare_template(obj)
        except TemplateDoesNotExist:
            return ""


class DateField(_HaystackDateField):
    FORMAT = "%Y-%m-%d"
    base_type = date

    def __init__(self, *args, **kwargs):
        self.format = kwargs.pop("format", None)
        super().__init__(*args, **kwargs)

    def prepare(self, obj):
        value = super().prepare(obj)
        if isinstance(value, self.base_type):
            return value.strftime(self.format or self.FORMAT)
        return value

    def convert(self, value):
        if isinstance(value, str) and (match := DATE_REGEX.match(value)):  # noqa F405
            return datetime_safe.date(  # noqa F405
                int(match.group("year")),
                int(match.group("month")),
                int(match.group("day")),
            ).strftime(self.FORMAT)
        return value


class DateTimeField(DateField):
    FORMAT = "%Y-%m-%d %H:%M:%S"
    base_format = datetime

    def convert(self, value):
        if isinstance(value, str) and (match := DATETIME_REGEX.match(value)):  # noqa F405
            return datetime_safe.datetime(  # noqa F405
                int(match.group("year")),
                int(match.group("month")),
                int(match.group("day")),
                int(match.group("hour")),
                int(match.group("minute")),
                int(match.group("second")),
            ).strftime(self.FORMAT)
        return value


# Для обратной совместимости с OpenSearch поле TextField приравнено к CharField
TextField = CharField  # noqa F405



from django import views
from django.views.decorators.csrf import csrf_exempt
from haystack import connections as haystack_connections
from haystack.exceptions import NotHandled
from haystack.utils.app_loading import haystack_get_models, haystack_load_apps

from django_solr import models


class FetchIndexesView(views.generic.RedirectView):
    pattern_name = "admin:django_solr_indexes_changelist"

    @csrf_exempt
    def post(self, request, *args, **kwargs):
        labels = haystack_load_apps()
        backends = haystack_connections.connections_info.keys()
        for label in labels:
            for using in backends:
                unified_index = haystack_connections[using].get_unified_index()
                for model in haystack_get_models(label):
                    try:
                        index = unified_index.get_index(model)
                    except NotHandled:
                        continue
                    hints = index.search().count()
                    record, is_created = models.Indexes.objects.get_or_create(
                        document=index.__class__.__name__,
                        defaults={
                            "model": index.model._meta.label,
                            "hints": hints,
                        },
                    )
                    if not is_created:
                        record.hints = hints
                        record.save()
        return super().post(request, *args, **kwargs)


class RebuildIndexView(views.generic.RedirectView):
    pass



import uuid

from django.db import models
from django.utils.translation import gettext_lazy as _


class Indexes(models.Model):
    guid = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    document = models.CharField(_("Document name"), max_length=255, db_index=True, editable=False)
    model = models.CharField(_("Model name"), max_length=255, db_index=True, editable=False)
    updated_at = models.DateTimeField(_("Date and time of last index"), null=True, blank=True, editable=False)
    hints = models.PositiveIntegerField(_("Count of hints in index"), default=0, editable=False)

    def __str__(self):
        return f"<{self.document}: {self.hints}>"

    class Meta:
        verbose_name = _("Index")
        verbose_name_plural = _("Indexes")
